///////////////////////////////////////////////////////////////////////////
//
//  APPritisesamo
//
//  Firmware ESP8266 ver 1.0
//
//  Gianluca Ghettini
//  
//
///////////////////////////////////////////////////////////////////////////

#include <ESP8266WiFi.h>
#include <Wire.h>
#include <PubSubClient.h>


/**********************************************************/



#define WIFI_SSID "ssidwifi"
#define WIFI_PASS ""***********""

#define mqtt_retry 5
#define mqtt_server "maqiatto.com"
#define mqtt_port 1883
#define mqtt_user "myusername"
#define mqtt_password "***********"
#define mqtt_topic "mytopic"
#define mqtt_userid "esp01"

#define PASSWORD_CH0 "***********"
#define PASSWORD_CH1 "***********"
#define PASSWORD_CH2 "***********"
#define PASSWORD_CH3 "***********"
#define PASSWORD_CH4 "***********"
#define PASSWORD_CH5 "***********"
#define PASSWORD_CH6 "***********"
#define PASSWORD_CH7 "***********"
#define PASSWORD_CH8 "***********"

/**********************************************************/



#define BUILTINLED 1

WiFiClient espClient;
PubSubClient mqttClient(espClient);

uint8_t gpioOut = 0b11111111;

void sendGpio(uint8_t gpio)
{
    Wire.beginTransmission(0x20);
    Wire.write(gpio);  
    Wire.endTransmission();  
}

void processRequest(byte* payload)
{    
      char cmd = payload[0];
      char chn = payload[1];
      char password[16];
      for(int p = 0; p < 16; p++) password[p] = payload[2 + p];

      uint8_t chnNum;
      if(chn == '0') { chnNum = 0b00000001; if(strncmp(password, PASSWORD_CH0, 16) != 0) return; }
      else if(chn == '1') { chnNum = 0b00000010; if(strncmp(password, PASSWORD_CH1, 16) != 0) return; }
      else if(chn == '2') { chnNum = 0b00000100; if(strncmp(password, PASSWORD_CH2, 16) != 0) return; }
      else if(chn == '3') { chnNum = 0b00001000; if(strncmp(password, PASSWORD_CH3, 16) != 0) return; }
      else if(chn == '4') { chnNum = 0b00010000; if(strncmp(password, PASSWORD_CH4, 16) != 0) return; }
      else if(chn == '5') { chnNum = 0b00100000; if(strncmp(password, PASSWORD_CH5, 16) != 0) return; }
      else if(chn == '6') { chnNum = 0b01000000; if(strncmp(password, PASSWORD_CH6, 16) != 0) return; }
      else if(chn == '7') { chnNum = 0b10000000; if(strncmp(password, PASSWORD_CH7, 16) != 0) return; }
      else return;

      if(cmd == 'P')
      {
            sendGpio(gpioOut & ~chnNum);
            delay(1000);
            sendGpio(gpioOut | chnNum);
            
            digitalWrite(BUILTINLED, HIGH);
            delay(100);
            digitalWrite(BUILTINLED, LOW);
            delay(100);  
      }
      else if(cmd == 'T')
      {    
            gpioOut = gpioOut ^ chnNum;
            sendGpio(gpioOut);

            digitalWrite(BUILTINLED, HIGH);
            delay(100);
            digitalWrite(BUILTINLED, LOW);
            delay(100); 
      }
      else
      {
            return;
      }
}

void callback(char* topic, byte* payload, unsigned int length)
{
      if(length != 18) return;
      processRequest(payload);
}

void setup()
{
      pinMode(BUILTINLED, OUTPUT);
      digitalWrite(BUILTINLED, HIGH);

      // setup PCF8574
      pinMode(0, OUTPUT);
      pinMode(2, OUTPUT);

      delay(5000);
      
      Wire.begin(0, 2); // SDA = GPIO0, SDL = GPIO2
      Wire.beginTransmission(0x20);
      Wire.write(0b11111111);  
      Wire.endTransmission();

      mqttClient.setServer(mqtt_server, mqtt_port);
      mqttClient.setCallback(callback); 
}

int connectWifiReliable()
{
      while(WiFi.status() != WL_CONNECTED)
      {      
              digitalWrite(BUILTINLED, HIGH);
        
              WiFi.mode(WIFI_STA);
              WiFi.begin(WIFI_SSID, WIFI_PASS);

              int count = 0;
              while(WiFi.status() != WL_CONNECTED)
              {
                      digitalWrite(BUILTINLED, HIGH);
                      delay(100);
                      digitalWrite(BUILTINLED, LOW);
                      delay(100);

                      if(count++ > 200)
                      {
                          digitalWrite(BUILTINLED, HIGH);
                          WiFi.disconnect();
                          while(1); // give up and reboot
                      }
              }
      }

      digitalWrite(BUILTINLED, LOW);
      return 1;
}


int connectMqttReliable()
{      
      if(mqttClient.connected()) return 1;

      int count = 0;
      while (!mqttClient.connected())
      {
          digitalWrite(BUILTINLED, HIGH);
          delay(50);
          digitalWrite(BUILTINLED, LOW);
          delay(50);
          digitalWrite(BUILTINLED, HIGH);
          delay(50);
          digitalWrite(BUILTINLED, LOW);
          delay(50);
          digitalWrite(BUILTINLED, HIGH);
          delay(50);
          digitalWrite(BUILTINLED, LOW);
          delay(50);

          if(!mqttClient.connect(mqtt_userid, mqtt_user, mqtt_password))
          {
              delay(1000);

              if(count++ > mqtt_retry)
              {
                  while(1); // give up and reboot
              }
          }


          mqttClient.subscribe(mqtt_topic, 0);    
      }

      return 1;
}


void loop()
{
    connectWifiReliable(); // connect wifi
    connectMqttReliable(); // connect mqtt
    mqttClient.loop();        
}
